
#define fuart_tx_buff_free (fuart_tx_buff_read_pos != ((fuart_tx_buff_write_pos + 1) & TX_BUFFER_MASK))
#define fuart_rx_buff_free (fuart_rx_buff_read_pos != ((fuart_rx_buff_write_pos + 1) & RX_BUFFER_MASK))

#if RX_ENABLE

#if RX_INTERRUPT

volatile uint8_t fuart_rx_buff_write_pos = 0;
volatile uint8_t fuart_rx_buff_read_pos = 0;
uint8_t fuart_rx_buff[RX_BUFFER_GR];
uint8_t fuart_rx_error = 0;

uint8_t fuart_received_data (void)
{
   return (fuart_rx_buff_write_pos != fuart_rx_buff_read_pos);
}

uint8_t fuart_getc(void) 
{
	uint8_t tmp_sreg= SREG; 
	cli();
	if (fuart_rx_buff_write_pos != fuart_rx_buff_read_pos)
	{
		uint8_t temp = fuart_rx_buff[fuart_rx_buff_read_pos];
		fuart_rx_buff_read_pos = (fuart_rx_buff_read_pos + 1) & RX_BUFFER_MASK;
		SREG = tmp_sreg; 
		return temp;
	}
	else
	{
		SREG = tmp_sreg;
		return 0;
	}
}

ISR(U0_RX_INTERRUPT) 
{
	if (fuart_rx_buff_free)
	{
		fuart_rx_buff[fuart_rx_buff_write_pos] = UDR; 
		fuart_rx_buff_write_pos = (fuart_rx_buff_write_pos + 1) & RX_BUFFER_MASK;
	}
	else
	{
		fuart_rx_error = 1;
	}
}
#else	// No RX_INTERRUPT

#define fuart_received_data() 	(UCSRA & (1<<RXC))

uint8_t fuart_getc(void)
{
    while (!(UCSRA & (1<<RXC))); 
    return UDR;
}
#endif	// RX_INTERRUPT

#endif	// RX_ENABLE


#if TX_ENABLE

#if TX_INTERRUPT

volatile uint8_t fuart_tx_buff_write_pos = 0;
volatile uint8_t fuart_tx_buff_read_pos = 0;
uint8_t fuart_tx_buff[TX_BUFFER_GR];

void  fuart_putc(uint8_t c)
{
	while (!( fuart_tx_buff_free ));
	uint8_t tmp_sreg = SREG; 
	cli();
	fuart_tx_buff[fuart_tx_buff_write_pos] = c;
	fuart_tx_buff_write_pos = (fuart_tx_buff_write_pos+1) & TX_BUFFER_MASK;
	if ( U0_UCSRA & (1<<UDRE))
	{
		if (fuart_tx_buff_write_pos != fuart_tx_buff_read_pos)
		{
			UDR = fuart_tx_buff[fuart_tx_buff_read_pos]; 
			fuart_tx_buff_read_pos = (fuart_tx_buff_read_pos + 1) & TX_BUFFER_MASK;
		}
	}
	SREG = tmp_sreg; 
}

ISR(U0_TX_INTERRUPT) 
{
	if (fuart_tx_buff_write_pos != fuart_tx_buff_read_pos)
	{
		UDR = fuart_tx_buff[fuart_tx_buff_read_pos]; 
		fuart_tx_buff_read_pos = (fuart_tx_buff_read_pos + 1) & TX_BUFFER_MASK;	
	}
}
#else // TX_INTERRUPT

void fuart_putc(unsigned char c)
{
    while ( !( U0_UCSRA & (1<<U0_UDRE)) );
    U0_UDR = c; 
}
#endif	// else TX_INTERRUPT

void fuart_puts (char *s)
{
    while (*s)
    { 
        fuart_putc(*s);
        s++;
    }
}
#endif	// TX_ENABLE


#if (RX_ENABLE || TX_ENABLE)
void fuart_ini(unsigned int ubrr)
{
	U0_UBRRH = (unsigned char)(ubrr>>8);
	U0_UBRRL = (unsigned char)ubrr;

#if TX_ENABLE		
	U0_UCSRB |= (1<<TXEN); 		// Transmitter Enable
	#if TX_INTERRUPT
		U0_UCSRB |= (1<<TXCIE); 	// TX Complete Interrupt Enable
	#endif
#endif
#if RX_ENABLE
	U0_UCSRB |= (1<<RXEN); 		// Receiver Enable
	#if RX_INTERRUPT
		U0_UCSRB |= (1<<RXCIE); 	// RX Complete Interrupt Enable
	#endif
#endif

#ifdef USE_URSEL		
	U0_UCSRC = (1<<URSEL)|(3<<UCSZ0);	// URSEL -> UCSRC ist gmeneint, 8 Bit
#else
	U0_UCSRC = (3<<UCSZ0);	
#endif
}
#endif	// (RX_ENABLE || #if TX_ENABLE)



