
#include <avr/interrupt.h>


#define TX_ENABLE		1
#define RX_ENABLE		0

#define TX_INTERRUPT	0
#define RX_INTERRUPT	0

//#define BAUD 			1200UL
//#define BAUD 			4800UL
#define BAUD 			9600UL
//#define BAUD 			38400UL

#define	TX_BUFFER_GR	8    // 2^n, (4,8...256)
#define	RX_BUFFER_GR	16    // 2^n, 

#define	TX_BUFFER_MASK		(TX_BUFFER_GR-1)
#define	RX_BUFFER_MASK		(RX_BUFFER_GR-1)

#ifndef F_CPU
/* In neueren Version der WinAVR/Mfile Makefile-Vorlage kann
   F_CPU im Makefile definiert werden, eine nochmalige Definition
   hier wuerde zu einer Compilerwarnung fuehren. Daher "Schutz" durch
   #ifndef/#endif 
 
   Dieser "Schutz" kann zu Debugsessions fhren, wenn AVRStudio 
   verwendet wird und dort eine andere, nicht zur Hardware passende 
   Taktrate eingestellt ist: Dann wird die folgende Definition 
   nicht verwendet, sondern stattdessen der Defaultwert (8 MHz?) 
   von AVRStudio - daher Ausgabe einer Warnung falls F_CPU
   noch nicht definiert: */
#warning "F_CPU war noch nicht definiert, wird nun nachgeholt mit 8000000"
#define F_CPU 8000000UL    // Systemtakt in Hz - Definition als unsigned long beachten >> Ohne ergeben Fehler in der Berechnung
#endif
 
// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden  (USART_Init (UBRR_VAL);)
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD) // Fehler in Promille, 1000 = kein Fehler.

#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch! 
#endif


#if  defined(__AVR_ATmega8__)  || defined(__AVR_ATmega16__) || defined(__AVR_ATmega32__) \
  || defined(__AVR_ATmega8515__) || defined(__AVR_ATmega8535__) \
  || defined(__AVR_ATmega323__)
 #define USE_URSEL					// UCSRC <-> UBRRH
 #define U0_TX_INTERRUPT	USART_TXC_vect
 #define U0_RX_INTERRUPT	USART_RXC_vect
 #define U0_UCSRA			UCSRA
 #define U0_UCSRB			UCSRB
 #define U0_UCSRC			UCSRC	// USART Control and Status Register C
 #define U0_UDRE			UDRE	// USART Data Register Empty
 #define U0_UDR				UDR		// USART I/O Data Register
 #define U0_UDRIE			UDRIE	// USART Data Register Empty Interrupt Enable
 #define U0_UBRRL			UBRRL	// USART Baud Rate Registers  UBRRL and UBRRH
 #define U0_UBRRH			UBRRH	// USART Baud Rate Registers  UBRRL and UBRRH

#elif defined(__AVR_ATtiny2313__)
 #define U0_TX_INTERRUPT	USART_TX_vect
 #define U0_RX_INTERRUPT	USART_RX_vect
 #define U0_UCSRA			UCSRA
 #define U0_UCSRB			UCSRB
 #define U0_UCSRC			UCSRC
 #define U0_UDRE			UDRE
 #define U0_UDR				UDR
 #define U0_UDRIE			UDRIE
 #define U0_UBRRL			UBRRL
 #define U0_UBRRH			UBRRH
 

#else
 #error "MC nicht unterstezt! (fuart.h)"
#endif
#include "fuart.c"


